@@ -57,8 +57,10 @@ public class CsvStatement implements Statement
 	private int maxRows = 0;
 	private int fetchSize = 1;
 	private int queryTimeout = Integer.MAX_VALUE;
+	private long timeoutMillis = Long.MAX_VALUE;
 	private int fetchDirection = ResultSet.FETCH_FORWARD;
 	private boolean closed;
+	protected boolean cancelled;
 
 	protected int resultSetType = ResultSet.TYPE_SCROLL_INSENSITIVE;
 
@@ -288,6 +290,9 @@ public class CsvStatement implements Statement
 			multipleParsers = null;
 		}
 
+		setTimeoutMillis();
+		cancelled = false;
+
 		SqlParser parser = new SqlParser();
 		try
 		{
@@ -512,10 +517,21 @@ public class CsvStatement implements Statement
 		}
 	}
 
+	public boolean isCancelled() throws SQLException
+	{
+		return cancelled;
+	}
+
 	@Override
 	public void cancel() throws SQLException
 	{
-		throw new SQLException(CsvResources.getString("methodNotSupported") + ": Statement.cancel()");
+		/*
+		 * JDBC documentation states that this method may be called
+		 * from a different thread to other JDBC calls.
+		 * Just set a flag here, so that the next JDBC call will
+		 * trigger the statement to close, avoiding any race condition.
+		 */
+		cancelled = true;
 	}
 
 	@Override
@@ -524,6 +540,26 @@ public class CsvStatement implements Statement
 		checkOpen();
 	}
 
+	/**
+	 * Set timestamp at which current query will throw an SQLTimeoutException.
+	 */
+	protected void setTimeoutMillis()
+	{
+		if (queryTimeout == 0 || queryTimeout == Integer.MAX_VALUE)
+			timeoutMillis = Long.MAX_VALUE;
+		else
+			timeoutMillis = System.currentTimeMillis() + queryTimeout * 1000;
+	}
+
+	/**
+	 * Get timestamp at which current query will throw an SQLTimeoutException.
+	 * @return timestamp in milliseconds, or MAX_VALUE if no timeout.
+	 */
+	public long getTimeoutMillis()
+	{
+		return timeoutMillis;
+	}
+
 	@Override
 	public boolean execute(String sql) throws SQLException
 	{
@@ -543,6 +579,9 @@ public class CsvStatement implements Statement
 			multipleParsers = null;
 		}
 
+		setTimeoutMillis();
+		cancelled = false;
+
 		/*
 		 * Execute one or more SQL statements.
 		 * The method getMoreResults() will be used to step through the results.
